@@ -5,11 +5,13 @@ import requests
 import time
 import datetime
 import typing
+import os
 from typing import Union, Optional, List, Dict
 Timestamp = Union[datetime.datetime, datetime.date, int, float]
 
 # API
-_URL_COIN_LIST = 'https://www.cryptocompare.com/api/data/coinlist/'
+_API_KEY_PARAMETER = ""
+_URL_COIN_LIST = 'https://www.cryptocompare.com/api/data/coinlist?'
 _URL_PRICE = 'https://min-api.cryptocompare.com/data/pricemulti?fsyms={}&tsyms={}'
 _URL_PRICE_MULTI = 'https://min-api.cryptocompare.com/data/pricemulti?fsyms={}&tsyms={}'
 _URL_PRICE_MULTI_FULL = 'https://min-api.cryptocompare.com/data/pricemultifull?fsyms={}&tsyms={}'
@@ -18,25 +20,27 @@ _URL_HIST_PRICE_DAY = 'https://min-api.cryptocompare.com/data/histoday?fsym={}&t
 _URL_HIST_PRICE_HOUR = 'https://min-api.cryptocompare.com/data/histohour?fsym={}&tsym={}&limit={}&e={}&toTs={}'
 _URL_HIST_PRICE_MINUTE = 'https://min-api.cryptocompare.com/data/histominute?fsym={}&tsym={}&limit={}&e={}&toTs={}'
 _URL_AVG = 'https://min-api.cryptocompare.com/data/generateAvg?fsym={}&tsym={}&e={}'
-_URL_EXCHANGES = 'https://www.cryptocompare.com/api/data/exchanges'
+_URL_EXCHANGES = 'https://www.cryptocompare.com/api/data/exchanges?'
 _URL_PAIRS = 'https://min-api.cryptocompare.com/data/pair/mapping/exchange?e={}'
 
 # DEFAULTS
-CURR = 'EUR'
+CURRENCY = 'EUR'
 LIMIT = 1440
 ###############################################################################
 
 
-def _query_cryptocompare(url: str, errorCheck: bool = True) -> Optional[Dict]:
+def _query_cryptocompare(url: str, errorCheck: bool = True, api_key: str = None) -> Optional[Dict]:
     """
     Query the url and return the result or None on failure.
 
     :param url: the url
     :param errorCheck: run extra error checks (default: True)
     :returns: respones, or nothing if errorCheck=True
+    :api_key: optional, if you want to add an API Key
     """
+    api_key_parameter = _set_api_key_parameter(api_key)
     try:
-        response = requests.get(url).json()
+        response = requests.get(url + api_key_parameter).json()
     except Exception as e:
         print('Error getting coin information. %s' % str(e))
         return None
@@ -72,6 +76,14 @@ def _format_timestamp(timestamp: Timestamp) -> int:
     return int(timestamp)
 
 
+def _set_api_key_parameter(api_key: str = None) -> str:
+    if api_key is None:
+        api_key = os.getenv('CRYPTOCOMPARE_API_KEY')
+    if api_key is not None:
+        _API_KEY = "&api_key={}".format(api_key)
+        return _API_KEY
+    return ""
+
 ###############################################################################
 
 
@@ -91,118 +103,118 @@ def get_coin_list(format: bool = False) -> Union[Dict, List, None]:
 # TODO: add option to filter json response according to a list of fields
 
 
-def get_price(coin: str, curr: str = CURR, full: bool = False) -> Optional[Dict]:
+def get_price(coin: str, currency: str = CURRENCY, full: bool = False) -> Optional[Dict]:
     """
-    Get the current price of a coin in a given currency.
+    Get the currencyent price of a coin in a given currency.
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param full: full response or just the price (default: False)
     :returns: dict of coin and currency price pairs
     """
     if full:
         return _query_cryptocompare(
             _URL_PRICE_MULTI_FULL.format(
-                _format_parameter(coin), _format_parameter(curr))
+                _format_parameter(coin), _format_parameter(currency))
         )
     if isinstance(coin, list):
         return _query_cryptocompare(
             _URL_PRICE_MULTI.format(_format_parameter(coin),
-                                    _format_parameter(curr))
+                                    _format_parameter(currency))
         )
     return _query_cryptocompare(
-        _URL_PRICE.format(coin, _format_parameter(curr))
+        _URL_PRICE.format(coin, _format_parameter(currency))
     )
 
 
-def get_historical_price(coin: str, curr: str = CURR, timestamp: Timestamp = time.time(),
+def get_historical_price(coin: str, currency: str = CURRENCY, timestamp: Timestamp = time.time(),
                          exchange: str = 'CCCAGG') -> Optional[Dict]:
     """
     Get the price of a coin in a given currency during a specific time.
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param timestamp: point in time
     :param exchange: the exchange to use
     :returns: dict of coin and currency price pairs
     """
     return _query_cryptocompare(
         _URL_HIST_PRICE.format(coin,
-                               _format_parameter(curr),
+                               _format_parameter(currency),
                                _format_timestamp(timestamp),
                                _format_parameter(exchange))
     )
 
 
-def get_historical_price_day(coin: str, curr: str = CURR, limit: int = LIMIT,
+def get_historical_price_day(coin: str, currency: str = CURRENCY, limit: int = LIMIT,
                              exchange: str = 'CCCAGG', toTs: Timestamp = time.time()) -> Optional[Dict]:
     """
     Get historical price (day).
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param limit: number of data points (max. 2000)
     :param exchange: exchange to use (default: 'CCCAGG')
     :param toTs: return data before this timestamp. (Unix epoch time or datetime object)
     :returns: dict of coin and currency price pairs
     """
     response = _query_cryptocompare(
-        _URL_HIST_PRICE_DAY.format(coin, _format_parameter(curr), limit, exchange, _format_timestamp(toTs)))
+        _URL_HIST_PRICE_DAY.format(coin, _format_parameter(currency), limit, exchange, _format_timestamp(toTs)))
     if response:
         return response['Data']
     return None
 
 
-def get_historical_price_hour(coin: str, curr: str = CURR, limit: int = LIMIT,
+def get_historical_price_hour(coin: str, currency: str = CURRENCY, limit: int = LIMIT,
                               exchange: str = 'CCCAGG', toTs: Timestamp = time.time()) -> Optional[Dict]:
     """
     Get historical price (hourly).
 
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param limit: number of data points (max. 2000)
     :param exchange: exchange to use (default: 'CCCAGG')
     :param toTs: return data before this timestamp. (Unix epoch time or datetime object)
     :returns: dict of coin and currency price pairs
     """
     response = _query_cryptocompare(
-        _URL_HIST_PRICE_HOUR.format(coin, _format_parameter(curr), limit, exchange, _format_timestamp(toTs)))
+        _URL_HIST_PRICE_HOUR.format(coin, _format_parameter(currency), limit, exchange, _format_timestamp(toTs)))
     if response:
         return response['Data']
     return None
 
 
-def get_historical_price_minute(coin: str, curr: str = CURR, limit: int = LIMIT,
+def get_historical_price_minute(coin: str, currency: str = CURRENCY, limit: int = LIMIT,
                                 exchange: str = 'CCCAGG', toTs: Timestamp = time.time()) -> Optional[Dict]:
     """
     Get historical price (minute).
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param limit: number of data points (max. 2000)
     :param exchange: exchange to use (default: 'CCCAGG')
     :param toTs: return data before this timestamp. (Unix epoch time or datetime object)
     :returns: dict of coin and currency price pairs
     """
     response = _query_cryptocompare(
-        _URL_HIST_PRICE_MINUTE.format(coin, _format_parameter(curr), limit, exchange, _format_timestamp(toTs)))
+        _URL_HIST_PRICE_MINUTE.format(coin, _format_parameter(currency), limit, exchange, _format_timestamp(toTs)))
     if response:
         return response['Data']
     return None
 
 
-def get_avg(coin: str, curr: str = CURR, exchange: str = 'CCCAGG') -> Optional[Dict]:
+def get_avg(coin: str, currency: str = CURRENCY, exchange: str = 'CCCAGG') -> Optional[Dict]:
     """
     Get the average price
 
     :param coin: symbolic name of the coin (e.g. BTC)
-    :param curr: short hand description of the currency (e.g. EUR)
+    :param currency: short hand description of the currency (e.g. EUR)
     :param exchange: exchange to use (default: 'CCCAGG')
     :returns: dict of coin and currency price pairs
     """
     response = _query_cryptocompare(_URL_AVG.format(
-        coin, curr, _format_parameter(exchange)))
+        coin, currency, _format_parameter(exchange)))
     if response:
         return response['RAW']
     return None
